@@ -27,14 +27,15 @@
    ports and the TCP ports. */
 
 #include <stdlib.h>
-#include <errno.h>
 #include <string.h>
 #include <ctype.h>
-#include <syslog.h>
 #include <assert.h>
+#include <stdint.h>
 
 #include <gensio/gensio.h>
+#ifndef GENSIO_EVENT_SER_MODEMSTATE
 #include <gensio/sergensio.h>
+#endif
 
 #include "port.h"
 #include "ser2net.h"
@@ -186,18 +187,18 @@ port_check_connect_backs(port_info_t *port)
 					   handle_net_event, netcon,
 					   &netcon->net);
 	    if (err) {
-		syslog(LOG_ERR, "Unable to allocate connect back port %s,"
-		       " addr %s: %s\n", port->name, netcon->remote_str,
-		       gensio_err_to_str(err));
+		seout.out(&seout, "Unable to allocate connect back port %s,"
+			  " addr %s: %s\n", port->name, netcon->remote_str,
+			  gensio_err_to_str(err));
 		continue;
 	    }
 	    err = gensio_open(netcon->net, connect_back_done, netcon);
 	    if (err) {
 		gensio_free(netcon->net);
 		netcon->net = NULL;
-		syslog(LOG_ERR, "Unable to open connect back port %s,"
-		       " addr %s: %s\n", port->name, netcon->remote_str,
-		       gensio_err_to_str(err));
+		seout.out(&seout, "Unable to open connect back port %s,"
+			  " addr %s: %s\n", port->name, netcon->remote_str,
+			  gensio_err_to_str(err));
 		continue;
 	    }
 	    port->num_waiting_connect_backs++;
@@ -239,8 +240,8 @@ handle_dev_read(port_info_t *port, int err, unsigned char *buf,
 	}
 
 	/* Got an error on the read, shut down the port. */
-	syslog(LOG_ERR, "dev read error for device on port %s: %s",
-	       port->name, gensio_err_to_str(err));
+	seout.out(&seout, "dev read error for device on port %s: %s",
+		  port->name, gensio_err_to_str(err));
 	shutdown_port(port, "dev read error");
     }
 
@@ -362,6 +363,9 @@ handle_dev_write_ready(port_info_t *port)
     so->unlock(port->lock);
 }
 
+static void handle_ser_modemstate(port_info_t *port, net_info_t *netcon);
+static void handle_ser_linestate(port_info_t *port, net_info_t *netcon);
+
 int
 handle_dev_event(struct gensio *io, void *user_data, int event, int err,
 		 unsigned char *buf, gensiods *buflen,
@@ -391,53 +395,23 @@ handle_dev_event(struct gensio *io, void *user_data, int event, int err,
     case GENSIO_EVENT_SER_MODEMSTATE:
 	so->lock(port->lock);
 	port->last_modemstate = *((unsigned int *) buf);
-	for_each_connection(port, netcon) {
-	    struct sergensio *sio;
-
-	    if (!netcon->net)
-		continue;
-	    sio = gensio_to_sergensio(netcon->net);
-	    if (!sio)
-		continue;
-
-	    /*
-	     * The 0xf below is non-standard, but the spec makes no
-	     * sense in this case.  From what I can tell, the
-	     * modemstate top 4 bits is the settings, and the bottom 4
-	     * bits is telling you what changed.  So you don't want to
-	     * report a value unless something changed, and only if it
-	     * was in the modemstate mask.
-	     */
-	    if (port->last_modemstate & netcon->modemstate_mask & 0xf)
-		sergensio_modemstate(sio, (port->last_modemstate &
-					   netcon->modemstate_mask));
-	}
+	for_each_connection(port, netcon)
+	    handle_ser_modemstate(port, netcon);
 	so->unlock(port->lock);
 	return 0;
 
     case GENSIO_EVENT_SER_LINESTATE:
 	so->lock(port->lock);
 	port->last_linestate = *((unsigned int *) buf);
-	for_each_connection(port, netcon) {
-	    struct sergensio *sio;
-
-	    if (!netcon->net)
-		continue;
-	    sio = gensio_to_sergensio(netcon->net);
-	    if (!sio)
-		continue;
-
-	    if (port->last_linestate & netcon->linestate_mask)
-		sergensio_linestate(sio, (port->last_linestate &
-					  netcon->linestate_mask));
-	}
+	for_each_connection(port, netcon)
+	    handle_ser_linestate(port, netcon);
 	so->unlock(port->lock);
 	return 0;
 
 #ifdef GENSIO_EVENT_PARMLOG
     case GENSIO_EVENT_PARMLOG: {
 	struct gensio_parmlog_data *d = (struct gensio_parmlog_data *) buf;
-	vsyslog(LOG_ERR, d->log, d->args);
+	seout.vout(&seout, d->log, d->args);
 	return 0;
     }
 #endif
@@ -495,8 +469,8 @@ dev_fd_write(port_info_t *port, struct gbuf *buf)
 
     err = gbuf_write(port, buf);
     if (err) {
-	syslog(LOG_ERR, "The dev write for port %s had error: %s",
-	       port->name, gensio_err_to_str(err));
+	seout.out(&seout, "The dev write for port %s had error: %s",
+		  port->name, gensio_err_to_str(err));
 	shutdown_port(port, "dev write error");
 	return;
     }
@@ -532,7 +506,7 @@ handle_dev_fd_devstr_write(port_info_t *port)
 
 /* Data is ready to read on the network port. */
 static gensiods
-handle_net_fd_read(net_info_t *netcon, struct gensio *net, int readerr,
+handle_net_fd_read(net_info_t *netcon, int readerr,
 		   unsigned char *buf, gensiods buflen)
 {
     port_info_t *port = netcon->port;
@@ -550,8 +524,8 @@ handle_net_fd_read(net_info_t *netcon, struct gensio *net, int readerr,
 	    reason = "network read close";
 	} else {
 	    /* Got an error on the read, shut down the port. */
-	    syslog(LOG_ERR, "read error for port %s: %s", port->name,
-		   gensio_err_to_str(readerr));
+	    seout.out(&seout, "read error for port %s: %s", port->name,
+		      gensio_err_to_str(readerr));
 	    reason = "network read error";
 	}
 	goto out_shutdown;
@@ -595,8 +569,8 @@ handle_net_fd_read(net_info_t *netcon, struct gensio *net, int readerr,
 
     err = gbuf_write(port, &port->net_to_dev);
     if (err) {
-	syslog(LOG_ERR, "The dev write(2) for port %s had error: %s",
-	       port->name, gensio_err_to_str(err));
+	seout.out(&seout, "The dev write(2) for port %s had error: %s",
+		  port->name, gensio_err_to_str(err));
 	shutdown_port(port, "dev write error");
 	rv = buflen;
 	goto out_unlock;
@@ -652,8 +626,8 @@ net_fd_write(port_info_t *port, net_info_t *netcon,
 	return -1;
     } else if (reterr) {
 	/* Some other bad error. */
-	syslog(LOG_ERR, "The network write for port %s had error: %s",
-	       port->name, gensio_err_to_str(reterr));
+	seout.out(&seout, "The network write for port %s had error: %s",
+		  port->name, gensio_err_to_str(reterr));
 	shutdown_one_netcon(netcon, "network write error");
 	return -1;
     }
@@ -686,7 +660,7 @@ finish_dev_to_net_write(port_info_t *port)
    if a write fails to complete, it is deactivated as soon as writing
    is available again. */
 static void
-handle_net_fd_write_ready(net_info_t *netcon, struct gensio *net)
+handle_net_fd_write_ready(net_info_t *netcon)
 {
     port_info_t *port = netcon->port;
     int rv = 1;
@@ -736,12 +710,436 @@ enum s2n_ser_ops {
     S2N_RTS
 };
 
+#ifdef GENSIO_ACONTROL_SER_BAUD
+static void
+handle_ser_modemstate(port_info_t *port, net_info_t *netcon)
+{
+    if (!netcon->net)
+	return;
+    /*
+     * The 0xf below is non-standard, but the spec makes no
+     * sense in this case.  From what I can tell, the
+     * modemstate top 4 bits is the settings, and the bottom 4
+     * bits is telling you what changed.  So you don't want to
+     * report a value unless something changed, and only if it
+     * was in the modemstate mask.
+     */
+    if (port->last_modemstate & netcon->modemstate_mask & 0xf) {
+	char s[20];
+	gensiods len = snprintf(s, sizeof(s), "%d", (port->last_modemstate &
+						     netcon->modemstate_mask));
+
+	gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		       GENSIO_CONTROL_SET,
+		       GENSIO_CONTROL_SER_MODEMSTATE, s, &len);
+    }
+}
+
+static void
+handle_ser_linestate(port_info_t *port, net_info_t *netcon)
+{
+    if (!netcon->net)
+	return;
+
+    if (port->last_linestate & netcon->linestate_mask) {
+	char s[20];
+	gensiods len = snprintf(s, sizeof(s), "%d", (port->last_linestate &
+						     netcon->linestate_mask));
+
+	gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		       GENSIO_CONTROL_SET,
+		       GENSIO_CONTROL_SER_LINESTATE, s, &len);
+    }
+}
+
+static void
+ser_control_set(struct gensio *io, int err,
+		const char *val, gensiods len, void *cb_data)
+{
+    port_info_t *port = gensio_get_user_data(io);
+    enum s2n_ser_ops op = (intptr_t) cb_data;
+    net_info_t *netcon;
+
+    so->lock(port->lock);
+    for_each_connection(port, netcon) {
+	struct gensio *io = netcon->net;
+
+	if (!io)
+	    continue;
+
+	switch (op) {
+	case S2N_BAUD:
+	    port->bps = strtol(val, NULL, 0);
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_BAUD, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+
+	case S2N_DATASIZE:
+	    port->bpc = strtol(val, NULL, 0);
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_DATASIZE, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+
+	case S2N_PARITY:
+	    if (strcmp(val, "none") == 0)
+		port->paritybits = 0;
+	    else
+		port->paritybits = 1;
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_PARITY, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+
+	case S2N_STOPBITS:
+	    port->stopbits = strtol(val, NULL, 0);
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_STOPBITS, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+
+	case S2N_FLOWCONTROL:
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_FLOWCONTROL, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+
+	case S2N_IFLOWCONTROL:
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_IFLOWCONTROL, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+
+	case S2N_BREAK:
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_SBREAK, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+
+	case S2N_DTR:
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_DTR, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+
+	case S2N_RTS:
+	    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+			    GENSIO_CONTROL_SET,
+			    GENSIO_ACONTROL_SER_RTS, val, strlen(val),
+			    NULL, NULL, NULL);
+	    break;
+	}
+    }
+    so->unlock(port->lock);
+}
+
+static void
+s2n_modemstate(net_info_t *netcon, struct gensio *io, unsigned int modemstate)
+{
+    port_info_t *port = netcon->port;
+    char s[20];
+    gensiods len;
+
+    netcon->modemstate_mask = modemstate;
+    len = snprintf(s, sizeof(s), "%d",
+		   port->last_modemstate & netcon->modemstate_mask);
+    gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		   GENSIO_CONTROL_SET,
+		   GENSIO_CONTROL_SER_MODEMSTATE, s, &len);
+}
+
+static void
+s2n_linestate(net_info_t *netcon, struct gensio *io, unsigned int linestate)
+{
+    port_info_t *port = netcon->port;
+    char s[20];
+    gensiods len;
+
+    netcon->linestate_mask = linestate;
+    len = snprintf(s, sizeof(s), "%d",
+		   port->last_linestate & netcon->linestate_mask);
+    gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		   GENSIO_CONTROL_SET,
+		   GENSIO_CONTROL_SER_LINESTATE, s, &len);
+}
+
+static void
+s2n_flowcontrol_state(net_info_t *netcon, bool val)
+{
+    char s[20];
+    gensiods len;
+
+    len = snprintf(s, sizeof(s), "%d", val);
+    gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		   GENSIO_CONTROL_SET,
+		   GENSIO_CONTROL_SER_FLOWCONTROL_STATE, s, &len);
+}
+
+static void
+s2n_flush(net_info_t *netcon, int val)
+{
+    char s[20];
+    gensiods len;
+
+    strncpy(s, gensio_onoff_to_str(val), sizeof(s) - 1);
+    len = strlen(s);
+
+    if (netcon->port->io)
+	gensio_control(netcon->port->io, GENSIO_CONTROL_DEPTH_FIRST,
+		       GENSIO_CONTROL_SET,
+		       GENSIO_CONTROL_SER_FLUSH, s, &len);
+
+    if (netcon->net)
+	gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		       GENSIO_CONTROL_SET,
+		       GENSIO_CONTROL_SER_FLUSH, s, &len);
+}
+
+static void
+s2n_baud(net_info_t *netcon, int baud)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    len = snprintf(s, sizeof(s), "%d", baud);
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_BAUD, s, len, ser_control_set,
+		    (void *) (long) S2N_BAUD, NULL);
+}
+
+static void
+s2n_datasize(net_info_t *netcon, int datasize)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    len = snprintf(s, sizeof(s), "%d", datasize);
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_DATASIZE, s, len, ser_control_set,
+		    (void *) (long) S2N_DATASIZE, NULL);
+}
+
+static void
+s2n_parity(net_info_t *netcon, int parity)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    len = snprintf(s, sizeof(s), "%s", gensio_parity_to_str(parity));
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_PARITY, s, len, ser_control_set,
+		    (void *) (long) S2N_PARITY, NULL);
+}
+
+static void
+s2n_stopbits(net_info_t *netcon, int stopbits)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    len = snprintf(s, sizeof(s), "%d", stopbits);
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_STOPBITS, s, len, ser_control_set,
+		    (void *) (long) S2N_STOPBITS, NULL);
+}
+
+static void
+s2n_flowcontrol(net_info_t *netcon, int flowcontrol)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    printf("A: %d\n", flowcontrol);
+    len = snprintf(s, sizeof(s), "%s", gensio_flowcontrol_to_str(flowcontrol));
+    printf("B: %ld %s\n", len, s);
+    int rv = gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_FLOWCONTROL, s, len, ser_control_set,
+		    (void *) (long) S2N_FLOWCONTROL, NULL);
+    printf("C: %d\n", rv);
+}
+
+static void
+s2n_iflowcontrol(net_info_t *netcon, int iflowcontrol)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    len = snprintf(s, sizeof(s), "%s", gensio_flowcontrol_to_str(iflowcontrol));
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_IFLOWCONTROL, s, len, ser_control_set,
+		    (void *) (long) S2N_IFLOWCONTROL, NULL);
+}
+
+static void
+s2n_sbreak(net_info_t *netcon, int breakv)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    len = snprintf(s, sizeof(s), "%s", gensio_onoff_to_str(breakv));
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_SBREAK, s, len, ser_control_set,
+		    (void *) (long) S2N_BREAK, NULL);
+}
+
+static void
+s2n_dtr(net_info_t *netcon, int dtr)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    len = snprintf(s, sizeof(s), "%s", gensio_onoff_to_str(dtr));
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_DTR, s, len, ser_control_set,
+		    (void *) (long) S2N_DTR, NULL);
+}
+
+static void
+s2n_rts(net_info_t *netcon, int rts)
+{
+    struct gensio *io = netcon->port->io;
+    char s[20];
+    gensiods len;
+
+    if (!io)
+	return;
+    len = snprintf(s, sizeof(s), "%s", gensio_onoff_to_str(rts));
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_RTS, s, len, ser_control_set,
+		    (void *) (long) S2N_RTS, NULL);
+}
+
+static void
+s2n_signature(net_info_t *netcon, struct gensio *io, char *sig,
+	      unsigned int sig_len)
+{
+    if (!io)
+	return;
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_SIGNATURE,
+		    sig, sig_len, NULL, NULL, NULL);
+}
+
+static void
+s2n_sync(net_info_t *netcon)
+{
+    struct gensio *io = netcon->port->io;
+    port_info_t *port = netcon->port;
+
+    if (!io)
+	return;
+    if (port->telnet_brk_on_sync)
+	gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+			GENSIO_CONTROL_SET,
+			GENSIO_CONTROL_SER_SEND_BREAK, "", 0,
+			NULL, NULL, NULL);
+}
+
+static void
+s2n_break(net_info_t *netcon)
+{
+    struct gensio *io = netcon->port->io;
+
+    if (!io)
+	return;
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_CONTROL_SER_SEND_BREAK, "", 0,
+		    NULL, NULL, NULL);
+}
+
+#else
+#include <gensio/sergensio.h>
+
+static void
+handle_ser_modemstate(port_info_t *port, net_info_t *netcon)
+{
+    struct sergensio *sio;
+
+    if (!netcon->net)
+	return;
+    sio = gensio_to_sergensio(netcon->net);
+    if (!sio)
+	return;
+
+    /*
+     * The 0xf below is non-standard, but the spec makes no
+     * sense in this case.  From what I can tell, the
+     * modemstate top 4 bits is the settings, and the bottom 4
+     * bits is telling you what changed.  So you don't want to
+     * report a value unless something changed, and only if it
+     * was in the modemstate mask.
+     */
+    if (port->last_modemstate & netcon->modemstate_mask & 0xf)
+	sergensio_modemstate(sio, (port->last_modemstate &
+				   netcon->modemstate_mask));
+}
+
+static void
+handle_ser_linestate(port_info_t *port, net_info_t *netcon)
+{
+    struct sergensio *sio;
+
+    if (!netcon->net)
+	return;
+    sio = gensio_to_sergensio(netcon->net);
+    if (!sio)
+	return;
+
+    if (port->last_linestate & netcon->linestate_mask)
+	sergensio_linestate(sio, (port->last_linestate &
+				  netcon->linestate_mask));
+}
+
 static void
 sergensio_val_set(struct sergensio *sio, int err,
 		  unsigned int val, void *cb_data)
 {
     port_info_t *port = sergensio_get_user_data(sio);
-    enum s2n_ser_ops op = (long) cb_data;
+    enum s2n_ser_ops op = (intptr_t) cb_data;
     net_info_t *netcon;
 
     so->lock(port->lock);
@@ -803,9 +1201,9 @@ sergensio_val_set(struct sergensio *sio, int err,
 }
 
 static void
-s2n_modemstate(net_info_t *netcon, struct sergensio *sio,
-	       unsigned int modemstate)
+s2n_modemstate(net_info_t *netcon, struct gensio *io, unsigned int modemstate)
 {
+    struct sergensio *sio = gensio_to_sergensio(io);
     port_info_t *port = netcon->port;
 
     if (!sio)
@@ -815,8 +1213,9 @@ s2n_modemstate(net_info_t *netcon, struct sergensio *sio,
 }
 
 static void
-s2n_linestate(net_info_t *netcon, struct sergensio *sio, unsigned int linestate)
+s2n_linestate(net_info_t *netcon, struct gensio *io, unsigned int linestate)
 {
+    struct sergensio *sio = gensio_to_sergensio(io);
     port_info_t *port = netcon->port;
 
     if (!sio)
@@ -826,7 +1225,7 @@ s2n_linestate(net_info_t *netcon, struct sergensio *sio, unsigned int linestate)
 }
 
 static void
-s2n_flowcontrol_state(net_info_t *netcon, struct sergensio *sio, bool val)
+s2n_flowcontrol_state(net_info_t *netcon, bool val)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -836,7 +1235,7 @@ s2n_flowcontrol_state(net_info_t *netcon, struct sergensio *sio, bool val)
 }
 
 static void
-s2n_flush(net_info_t *netcon, struct sergensio *sio, int val)
+s2n_flush(net_info_t *netcon, int val)
 {
     struct sergensio *port_rsio = gensio_to_sergensio(netcon->port->io);
     struct sergensio *net_rsio = gensio_to_sergensio(netcon->net);
@@ -849,7 +1248,7 @@ s2n_flush(net_info_t *netcon, struct sergensio *sio, int val)
 }
 
 static void
-s2n_baud(net_info_t *netcon, struct sergensio *sio, int baud)
+s2n_baud(net_info_t *netcon, int baud)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -860,7 +1259,7 @@ s2n_baud(net_info_t *netcon, struct sergensio *sio, int baud)
 }
 
 static void
-s2n_datasize(net_info_t *netcon, struct sergensio *sio, int datasize)
+s2n_datasize(net_info_t *netcon, int datasize)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -871,7 +1270,7 @@ s2n_datasize(net_info_t *netcon, struct sergensio *sio, int datasize)
 }
 
 static void
-s2n_parity(net_info_t *netcon, struct sergensio *sio, int parity)
+s2n_parity(net_info_t *netcon, int parity)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -882,7 +1281,7 @@ s2n_parity(net_info_t *netcon, struct sergensio *sio, int parity)
 }
 
 static void
-s2n_stopbits(net_info_t *netcon, struct sergensio *sio, int stopbits)
+s2n_stopbits(net_info_t *netcon, int stopbits)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -893,7 +1292,7 @@ s2n_stopbits(net_info_t *netcon, struct sergensio *sio, int stopbits)
 }
 
 static void
-s2n_flowcontrol(net_info_t *netcon, struct sergensio *sio, int flowcontrol)
+s2n_flowcontrol(net_info_t *netcon, int flowcontrol)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -904,7 +1303,7 @@ s2n_flowcontrol(net_info_t *netcon, struct sergensio *sio, int flowcontrol)
 }
 
 static void
-s2n_iflowcontrol(net_info_t *netcon, struct sergensio *sio, int iflowcontrol)
+s2n_iflowcontrol(net_info_t *netcon, int iflowcontrol)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -915,7 +1314,7 @@ s2n_iflowcontrol(net_info_t *netcon, struct sergensio *sio, int iflowcontrol)
 }
 
 static void
-s2n_sbreak(net_info_t *netcon, struct sergensio *sio, int breakv)
+s2n_sbreak(net_info_t *netcon, int breakv)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -926,7 +1325,7 @@ s2n_sbreak(net_info_t *netcon, struct sergensio *sio, int breakv)
 }
 
 static void
-s2n_dtr(net_info_t *netcon, struct sergensio *sio, int dtr)
+s2n_dtr(net_info_t *netcon, int dtr)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -936,7 +1335,7 @@ s2n_dtr(net_info_t *netcon, struct sergensio *sio, int dtr)
 }
 
 static void
-s2n_rts(net_info_t *netcon, struct sergensio *sio, int rts)
+s2n_rts(net_info_t *netcon, int rts)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -946,9 +1345,10 @@ s2n_rts(net_info_t *netcon, struct sergensio *sio, int rts)
 }
 
 static void
-s2n_signature(net_info_t *netcon, struct sergensio *sio, char *sig,
+s2n_signature(net_info_t *netcon, struct gensio *io, char *sig,
 	      unsigned int sig_len)
 {
+    struct sergensio *sio = gensio_to_sergensio(io);
     port_info_t *port = netcon->port;
 
     if (!sio)
@@ -962,7 +1362,7 @@ s2n_signature(net_info_t *netcon, struct sergensio *sio, char *sig,
 }
 
 static void
-s2n_sync(net_info_t *netcon, struct sergensio *sio)
+s2n_sync(net_info_t *netcon)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
     port_info_t *port = netcon->port;
@@ -974,7 +1374,7 @@ s2n_sync(net_info_t *netcon, struct sergensio *sio)
 }
 
 static void
-s2n_break(net_info_t *netcon, struct gensio *io)
+s2n_break(net_info_t *netcon)
 {
     struct sergensio *rsio = gensio_to_sergensio(netcon->port->io);
 
@@ -982,6 +1382,7 @@ s2n_break(net_info_t *netcon, struct gensio *io)
 	return;
     sergensio_send_break(rsio);
 }
+#endif
 
 static int
 handle_net_event(struct gensio *net, void *user_data, int event, int err,
@@ -989,91 +1390,98 @@ handle_net_event(struct gensio *net, void *user_data, int event, int err,
 		 const char *const *auxdata)
 {
     net_info_t *netcon = user_data;
-    gensiods len= 0;
+    gensiods len = 0;
 
     if (buflen)
 	len = *buflen;
 
     switch (event) {
     case GENSIO_EVENT_READ:
-	len = handle_net_fd_read(netcon, net, err, buf, len);
+	len = handle_net_fd_read(netcon, err, buf, len);
 	if (buflen)
 	    *buflen = len;
 	return 0;
 
     case GENSIO_EVENT_WRITE_READY:
-	handle_net_fd_write_ready(netcon, net);
+	handle_net_fd_write_ready(netcon);
 	return 0;
 
     case GENSIO_EVENT_SEND_BREAK:
-	s2n_break(netcon, net);
+	s2n_break(netcon);
 	return 0;
 
+#ifdef GENSIO_EVENT_SER_MODEMSTATE_MASK
+    case GENSIO_EVENT_SER_MODEMSTATE_MASK:
+#else
     case GENSIO_EVENT_SER_MODEMSTATE:
-	s2n_modemstate(netcon, gensio_to_sergensio(net),
-		       *((unsigned int *) buf));
+#endif
+	s2n_modemstate(netcon, net, *((unsigned int *) buf));
 	return 0;
 
+#ifdef GENSIO_EVENT_SER_LINESTATE_MASK
+    case GENSIO_EVENT_SER_LINESTATE_MASK:
+#else
     case GENSIO_EVENT_SER_LINESTATE:
-	s2n_linestate(netcon, gensio_to_sergensio(net), *((unsigned int *) buf));
+#endif
+	s2n_linestate(netcon, net, *((unsigned int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_SIGNATURE:
-	s2n_signature(netcon, gensio_to_sergensio(net), (char *) buf, len);
+	s2n_signature(netcon, net, (char *) buf, len);
 	return 0;
 
     case GENSIO_EVENT_SER_FLOW_STATE:
-	s2n_flowcontrol_state(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_flowcontrol_state(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_FLUSH:
-	s2n_flush(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_flush(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_SYNC:
-	s2n_sync(netcon, gensio_to_sergensio(net));
+	s2n_sync(netcon);
 	return 0;
 
     case GENSIO_EVENT_SER_BAUD:
-	s2n_baud(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_baud(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_DATASIZE:
-	s2n_datasize(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_datasize(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_PARITY:
-	s2n_parity(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_parity(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_STOPBITS:
-	s2n_stopbits(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_stopbits(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_FLOWCONTROL:
-	s2n_flowcontrol(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_flowcontrol(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_IFLOWCONTROL:
-	s2n_iflowcontrol(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_iflowcontrol(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_SBREAK:
-	s2n_sbreak(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_sbreak(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_DTR:
-	s2n_dtr(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_dtr(netcon, *((int *) buf));
 	return 0;
 
     case GENSIO_EVENT_SER_RTS:
-	s2n_rts(netcon, gensio_to_sergensio(net), *((int *) buf));
+	s2n_rts(netcon, *((int *) buf));
 	return 0;
 
 #ifdef GENSIO_EVENT_PARMLOG
     case GENSIO_EVENT_PARMLOG: {
 	struct gensio_parmlog_data *d = (struct gensio_parmlog_data *) buf;
-	vsyslog(LOG_ERR, d->log, d->args);
+	seout.vout(&seout, d->log, d->args);
 	return 0;
     }
 #endif
@@ -1190,7 +1598,7 @@ port_dev_open_done(struct gensio *io, int err, void *cb_data)
 
     if (port->devstr)
 	gbuf_free(port->devstr);
-    port->devstr = process_str_to_buf(port, NULL, port->openstr);
+    port->devstr = process_str_to_buf(port, NULL, port->openstr, &seout);
     if (port->devstr)
 	port->dev_write_handler = handle_dev_fd_devstr_write;
     else
@@ -1200,7 +1608,7 @@ port_dev_open_done(struct gensio *io, int err, void *cb_data)
 	gensio_set_write_callback_enable(port->io, true);
     gensio_set_read_callback_enable(port->io, true);
 
-    setup_trace(port);
+    setup_trace(port, &seout);
 
     port_start_timer(port);
 
@@ -1229,8 +1637,8 @@ port_dev_enable(port_info_t *port)
     err = gensio_control(port->io, GENSIO_CONTROL_DEPTH_ALL, false,
 			 GENSIO_CONTROL_NODELAY, auxdata, NULL);
     if (err)
-	syslog(LOG_ERR, "Could not enable NODELAY on port %s: %s",
-	       port->name, gensio_err_to_str(err));
+	seout.out(&seout, "Could not enable NODELAY on port %s: %s",
+		  port->name, gensio_err_to_str(err));
 
     return 0;
 }
@@ -1245,12 +1653,12 @@ setup_port(port_info_t *port, net_info_t *netcon)
     err = gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_ALL, false,
 			 GENSIO_CONTROL_NODELAY, auxdata, NULL);
     if (err)
-	syslog(LOG_ERR, "Could not enable NODELAY on socket %s: %s",
-	       port->name, gensio_err_to_str(err));
+	seout.out(&seout, "Could not enable NODELAY on socket %s: %s",
+		  port->name, gensio_err_to_str(err));
 
     if (netcon->banner)
 	gbuf_free(netcon->banner);
-    netcon->banner = process_str_to_buf(port, netcon, port->bannerstr);
+    netcon->banner = process_str_to_buf(port, netcon, port->bannerstr, &seout);
 
     if (num_connected_net(port) == 1 && (!port->connbacks || !port->io_open)) {
 	/* We are first, set things up on the device. */
