@@ -27,25 +27,18 @@
    parameters, initializes everything, then starts the select loop. */
 
 #include <stdio.h>
-#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
-#include <syslog.h>
-#include <sys/types.h>
-#include <unistd.h>
-#include <errno.h>
 
-#include <gensio/selector.h>
+#include <gensio/gensio_os_funcs_public.h>
 
 #include "ser2net.h"
 #include "readconfig.h"
 #include "controller.h"
 #include "dataxfer.h"
 #include "led.h"
+#include "fileio.h"
 
-static char *config_file = SYSCONFDIR "/ser2net/ser2net.yaml";
-static bool config_file_set = false;
-static char *old_config_file = SYSCONFDIR "/ser2net.conf";
 bool admin_port_from_cmdline = false;
 char *admin_port = NULL; /* Can be set from readconfig, too. */
 static char *pid_file = NULL;
@@ -53,23 +46,191 @@ static int detach = 1;
 int ser2net_debug = 0;
 int ser2net_debug_level = 0;
 volatile int in_shutdown = 0;
-#ifdef USE_PTHREADS
-#include <pthread.h>
-int num_threads = 1;
-struct thread_info {
-    pthread_t id;
-};
-struct thread_info *threads;
+static unsigned int num_threads = 1;
+
+char *confdir;
+char *authdir;
+char *admin_authdir;
+char *keyfile;
+char *certfile;
+
+static char *config_file;
+bool config_file_set;
+
+static char *
+alloc_vsprintf(const char *fmt, va_list va)
+{
+    va_list va2;
+    int len;
+    char c[1], *str;
+
+    va_copy(va2, va);
+    len = vsnprintf(c, 0, fmt, va);
+    str = malloc(len + 1);
+    if (str)
+        vsnprintf(str, len + 1, fmt, va2);
+    va_end(va2);
+    return str;
+}
+
+static char *
+alloc_sprintf(const char *fmt, ...)
+{
+    va_list va;
+    char *s;
+
+    va_start(va, fmt);
+    s = alloc_vsprintf(fmt, va);
+    va_end(va);
+    return s;
+}
+
+#ifdef _WIN32
+#include <windows.h>
+
+static int
+drop_last_diritem(char *dir)
+{
+    char *s, *s2;
+
+    s = strrchr(dir, '/');
+    s2 = strrchr(dir, '\\');
+    if (!s && !s2)
+	return 0;
+    if (s && s2) {
+	if (s < s2)
+	    s = s2;
+	} else if (s2) {
+	s = s2;
+    }
+    *s = '\0';
+    return 1;
+}
+
+static char *
+get_basedir(void)
+{
+    char dir[256];
+    DWORD rv;
+
+    rv = GetModuleFileNameA(NULL, dir, sizeof(dir));
+    if (rv == 0)
+	return NULL;
+    if (!drop_last_diritem(dir))
+	return NULL;
+    if (!drop_last_diritem(dir))
+	return NULL;
+    return strdup(dir);
+}
+
+static int
+setup_paths_base(void)
+{
+    char *basedir = get_basedir();
+
+    if (!basedir) {
+	fprintf(stderr, "Unable to allocate base dir\n");
+	return 1;
+    }
+    if (!confdir) {
+	confdir = alloc_sprintf("%s%setc%sser2net", basedir, DIRSEPS, DIRSEPS);
+	if (!confdir) {
+	    fprintf(stderr, "Unable to allocate confdir\n");
+	    goto out_err;
+	}
+    }
+    if (!authdir) {
+	authdir = alloc_sprintf("%s%sshare%sser2net%sauth", basedir, DIRSEPS, DIRSEPS, DIRSEPS);
+	if (!confdir) {
+	    fprintf(stderr, "Unable to allocate authdir\n");
+	    goto out_err;
+	}
+    }
+    free(basedir);
+    return 0;
+
+ out_err:
+    free(basedir);
+    return 1;
+}
+
+#else
+static int
+setup_paths_base(void)
+{
+    if (!confdir) {
+	confdir = alloc_sprintf("%s%sser2net",
+				SYSCONFDIR, DIRSEPS);
+	if (!confdir) {
+	    fprintf(stderr, "Unable to allocate authdir\n");
+	    return 1;
+	}
+    }
+    if (!authdir) {
+	authdir = alloc_sprintf("%s%sser2net%sauth",
+				DATAROOT, DIRSEPS, DIRSEPS);
+	if (!authdir) {
+	    fprintf(stderr, "Unable to allocate authdir\n");
+	    return 1;
+	}
+    }
+    return 0;
+}
 #endif
 
+static int
+setup_paths(void)
+{
+    if (setup_paths_base())
+	return 1;
 
-struct selector_s *ser2net_sel;
+    if (!admin_authdir) {
+	admin_authdir = alloc_sprintf("%s%sauth", confdir, DIRSEPS);
+	if (!admin_authdir) {
+	    fprintf(stderr, "Unable to allocate admin authdir\n");
+	    goto out_err;
+	}
+    }
+    if (!keyfile) {
+	keyfile = alloc_sprintf("%s%sser2net.key", confdir, DIRSEPS);
+	if (!keyfile) {
+	    fprintf(stderr, "Unable to allocate keyfile\n");
+	    goto out_err;
+	}
+    }
+    if (!certfile) {
+	certfile = alloc_sprintf("%s%sser2net.crt", confdir, DIRSEPS);
+	if (!certfile) {
+	    fprintf(stderr, "Unable to allocate certfile\n");
+	    goto out_err;
+	}
+    }
+    if (!config_file && !config_file_set) {
+	config_file = alloc_sprintf("%s%sser2net.yaml", confdir, DIRSEPS);
+	if (!config_file) {
+	    fprintf(stderr, "Unable to allocate config file name\n");
+	    goto out_err;
+	}
+    }
+    return 0;
+
+ out_err:
+    return 1;
+}
+
+struct s2n_threadinfo {
+    struct gensio_thread *gthread;
+    struct gensio_waiter *waiter;
+    struct gensio_runner *runner;
+} *threads;
+
+struct gensio_os_proc_data *procdata;
 struct gensio_os_funcs *so;
 char *rfc2217_signature = "ser2net";
 
 static char *help_string =
 "%s: Valid parameters are:\n"
-"  -c <config file> - use a config file besides /etc/ser2net/ser2net.yaml\n"
+"  -c <config file> - use a config file besides %s\n"
 "  -C <config line> - Handle a single configuration line.  This may be\n"
 "     specified multiple times for multiple lines.  This is just like a\n"
 "     line in the config file.  This disables the default config file,\n"
@@ -81,9 +242,7 @@ static char *help_string =
 "  -d - Don't detach and send debug I/O to standard output\n"
 "  -l - Increase the debugging level\n"
 "  -u - Disable UUCP locking\n"
-#ifdef USE_PTHREADS
 "  -t <num threads> - Use the given number of threads, default 1\n"
-#endif
 "  -b - unused (was Do CISCO IOS baud-rate negotiation, instead of RFC2217)\n"
 "  -v - print the program's version and exit\n"
 "  -s - specify a default signature for RFC2217 protocol\n"
@@ -92,54 +251,69 @@ static char *help_string =
 "       string.  This disables the default config file, you must specify -c\n"
 "       after the last -Y.  The config file will be processed first, if it\n"
 "       is specified, then the -Y strings in order, as if they are one\n"
-"       continguous file.  '#' characters outside of quotes will be converted\n"
+"       contiguous file.  '#' characters outside of quotes will be converted\n"
 "       to newlines to make things easier to handle.  Each -Y will be\n"
 "       terminated with a newline automatically.\n";
 
 static char **config_lines;
 static unsigned int num_config_lines;
 
-static bool
-str_endswith(const char *str, const char *end)
-{
-    unsigned int slen = strlen(str);
-    unsigned int elen = strlen(end);
+#ifndef WIN32
+#include <syslog.h>
+#endif
 
-    if (elen > slen)
-	return false;
-    if (strcmp(end, str + slen - elen) == 0)
-	return true;
-    return false;
+static int
+stderr_evout(struct absout *e, const char *str, va_list ap)
+{
+    char buf[1024];
+
+    vsnprintf(buf, sizeof(buf), str, ap);
+#ifndef WIN32
+    syslog(LOG_ERR, "%s", buf);
+#endif
+    fprintf(stderr, "%s\n", buf);
+    return 0;
+}
+static int
+stderr_eout(struct absout *e, const char *str, ...)
+{
+    va_list ap;
+
+    va_start(ap, str);
+    stderr_evout(e, str, ap);
+    va_end(ap);
+    return 0;
+}
+
+#ifndef WIN32
+#include <errno.h>
+#include <unistd.h>
+
+static int
+syslog_evout(struct absout *e, const char *str, va_list ap)
+{
+    char buf[1024];
+
+    vsnprintf(buf, sizeof(buf), str, ap);
+    syslog(LOG_ERR, "%s", buf);
+    return 0;
 }
 
 static int
 syslog_eout(struct absout *e, const char *str, ...)
 {
     va_list ap;
-    char buf[1024];
 
     va_start(ap, str);
-    vsnprintf(buf, sizeof(buf), str, ap);
+    syslog_evout(e, str, ap);
     va_end(ap);
-    syslog(LOG_ERR, "%s", buf);
     return 0;
 }
-struct absout syslog_absout = { .out = syslog_eout };
 
-static int
-stderr_eout(struct absout *e, const char *str, ...)
-{
-    va_list ap;
-    char buf[1024];
-
-    va_start(ap, str);
-    vsnprintf(buf, sizeof(buf), str, ap);
-    va_end(ap);
-    syslog(LOG_ERR, "%s", buf);
-    fprintf(stderr, "%s\n", buf);
-    return 0;
-}
-static struct absout stderr_absout = { .out = stderr_eout };
+struct absout seout = {
+    .out = syslog_eout,
+    .vout = syslog_evout
+};
 
 static int
 gensio_log_level_to_syslog(int gloglevel)
@@ -152,6 +326,7 @@ gensio_log_level_to_syslog(int gloglevel)
     case GENSIO_LOG_WARNING:
 	return LOG_WARNING;
     case GENSIO_LOG_INFO:
+    default:
 	return LOG_INFO;
     }
     return LOG_ERR;
@@ -166,89 +341,97 @@ do_gensio_log(const char *name, struct gensio_loginfo *i)
     syslog(gensio_log_level_to_syslog(i->level), "%s: %s", name, buf);
 }
 
-static FILE *
-fopen_config_file(bool *is_yaml, char **rfilename)
+static void
+ser2net_gensio_logger(struct gensio_os_funcs *o, enum gensio_log_levels level,
+		      const char *log, va_list args)
 {
-    FILE *instream = fopen(config_file, "r");
+    int priority = gensio_log_level_to_syslog(level);
 
-    if (!instream) {
-	if (config_file_set) {
-	    syslog(LOG_ERR, "Unable to open config file '%s': %m",
-		   config_file);
-	    return NULL;
-	}
-	instream = fopen(old_config_file, "r");
-	if (!instream) {
-	    syslog(LOG_ERR, "Unable to open config file '%s' or old one"
-		   " '%s': %m", config_file, old_config_file);
-	    return NULL;
-	}
-	*rfilename = old_config_file;
-	*is_yaml = false;
+    vsyslog(priority, log, args);
+}
+
+#else
+
+#define SIGUSR1 0
+
+struct absout seout = {
+    .out = stderr_eout,
+    .vout = stderr_evout
+};
+
+void
+do_gensio_log(const char *name, struct gensio_loginfo *i)
+{
+    char buf[256];
+
+    vsnprintf(buf, sizeof(buf), i->str, i->args);
+    fprintf(stderr, "%s: %s: %s\n", gensio_log_level_to_str(i->level), name, buf);
+}
+
+static void
+ser2net_gensio_logger(struct gensio_os_funcs *o, enum gensio_log_levels level,
+		      const char *log, va_list args)
+{
+    char buf[256];
+
+    vsnprintf(buf, sizeof(buf), log, args);
+    fprintf(stderr, "%s: %s\n", gensio_log_level_to_str(level), buf);
+}
+
+#endif
+
+static struct absout stderr_absout = {
+    .out = stderr_eout,
+    .vout = stderr_evout
+};
+
+static ftype *
+fopen_config_file(char **rfilename, struct absout *eout)
+{
+    int err;
+    ftype *instream;
+
+    err = f_open(config_file, DO_READ, 0, &instream);
+    if (err) {
+	eout->out(eout, "Unable to open config file '%s': %s",
+		  config_file, gensio_err_to_str(err));
+	return NULL;
     } else {
 	*rfilename = config_file;
-	*is_yaml = str_endswith(config_file, ".yaml");
     }
 
-    if (!*is_yaml) {
-	char in_buf[100];
-
-	in_buf[0] = '\0';
-	if (fgets(in_buf, sizeof(in_buf) - 1, instream) != NULL)
-	    *is_yaml = strncmp(in_buf, "%YAML", 5) == 0;
-	fseek(instream, 0, SEEK_SET);
-    }
-    if (!*is_yaml)
-	fprintf(stderr,
-	  "ser2net:WARNING: Using old config file format, this will go away\n"
-	  "soon.  Please switch to the yaml-based format.\n");
     return instream;
 }
 
 int
 reread_config_file(const char *reqtype, struct absout *eout)
 {
-    int rv = ENOENT;
+    int rv = GE_NOTFOUND;
 
     if (config_file) {
-	FILE *instream = NULL;
+	ftype *instream = NULL;
 	char *filename;
-	bool is_yaml;
 
-	syslog(LOG_INFO, "Got %s, re-reading configuration", reqtype);
+	seout.out(&seout, "Got %s, re-reading configuration", reqtype);
 	readconfig_init();
 
-	instream = fopen_config_file(&is_yaml, &filename);
+	instream = fopen_config_file(&filename, &seout);
 	if (!instream)
 	    goto out;
 
 	if (!admin_port_from_cmdline)
 	    controller_shutdown();
-	if (is_yaml)
-	    rv = yaml_readconfig(instream, filename,
-				 config_lines, num_config_lines, eout);
-	else
-	    rv = readconfig(instream);
-	fclose(instream);
+	rv = yaml_readconfig(instream, filename,
+			     config_lines, num_config_lines, eout);
+	f_close(instream);
 
 	if (!rv)
-	    apply_new_ports(&syslog_absout);
+	    apply_new_ports(&seout);
     }
  out:
     return rv;
 }
 
-void
-write_ignore_fail(int fd, const char *data, size_t count)
-{
-    ssize_t written;
-
-    while ((written = write(fd, data, count)) > 0) {
-	data += written;
-	count -= written;
-    }
-}
-
 void
 add_usec_to_time(gensio_time *tv, int usec)
 {
@@ -327,400 +510,279 @@ scan_int(const char *str)
 static void
 arg_error(char *name)
 {
-    fprintf(stderr, help_string, name);
-    exit(1);
+    if (setup_paths())
+	return;
+    fprintf(stderr, help_string, name, config_file);
+    printf("\n");
+    printf("  config file: %s\n", config_file);
+    printf("  config dir: %s\n", confdir);
+    printf("  user auth dir: %s\n", authdir);
+    printf("  admin auth dir: %s\n", admin_authdir);
+    printf("  keyfile: %s\n", keyfile);
+    printf("  certfile: %s\n", certfile);
 }
 
-static void
+static int
 make_pidfile(void)
 {
-    FILE *fpidfile;
+#ifndef WIN32
+    int rv, len;
+    ftype *fpidfile;
+    char buf[20];
+
     if (!pid_file)
-	return;
-    fpidfile = fopen(pid_file, "w");
-    if (!fpidfile) {
-	syslog(LOG_WARNING,
-	       "Error opening pidfile '%s': %m, pidfile not created",
-	       pid_file);
+	return 0;
+    rv = f_open(pid_file, DO_WRITE | DO_CREATE, 0644, &fpidfile);
+    if (rv) {
+	seout.out(&seout,
+		  "Error opening pidfile '%s': %m, pidfile not created: %s",
+		  pid_file, gensio_err_to_str(rv));
 	pid_file = NULL;
-	return;
+	return rv;
     }
-    fprintf(fpidfile, "%d\n", getpid());
-    fclose(fpidfile);
-}
-
-static int dummyrv; /* Used to ignore return values of read() and write(). */
-
-/* Used to reliably deliver signals to a thread.  This is a pipe that
-   we write to from a signal handler to make sure it wakes up. */
-static int sig_fd_alert = -1;
-static int sig_fd_watch = -1;
-static volatile int reread_config = 0; /* Did I get a HUP signal? */
-static volatile int term_prog = 0; /* Did I get an INT signal? */
-
-static void
-sig_wake_selector(void)
-{
-    char dummy = 0;
-
-    dummyrv = write(sig_fd_alert, &dummy, 1);
+    len = snprintf(buf, sizeof(buf), "%d\n", getpid());
+    f_write(fpidfile, buf, len, NULL);
+    f_close(fpidfile);
+#endif
+    return 0;
 }
 
 static void
-sighup_handler(int sig)
+cleanup_pidfile(void)
 {
-    reread_config = 1;
-    sig_wake_selector();
+#ifndef WIN32
+    if (pid_file)
+	unlink(pid_file);
+#endif
 }
 
-static void
-sigint_handler(int sig)
+void
+do_detach(void)
 {
-    term_prog = 1;
-    sig_wake_selector();
+#ifndef WIN32
+    if (detach) {
+	int pid;
+
+	/* Detach from the calling terminal. */
+	openlog("ser2net", LOG_PID | LOG_CONS, LOG_DAEMON);
+	syslog(LOG_NOTICE, "ser2net startup");
+	if ((pid = fork()) > 0) {
+	    exit(0);
+	} else if (pid < 0) {
+	    seout.out(&seout, "Error forking first fork: %s", strerror(errno));
+	    exit(1);
+	} else {
+	    /* setsid() is necessary if we really want to demonize */
+	    setsid();
+	    /* Second fork to really daemonize me. */
+	    if ((pid = fork()) > 0) {
+		exit(0);
+	    } else if (pid < 0) {
+		seout.out(&seout, "Error forking second fork: %s",
+			  strerror(errno));
+		exit(1);
+	    }
+	}
+
+	/* Close all my standard I/O. */
+	if (chdir("/") < 0) {
+	    seout.out(&seout, "unable to chdir to '/': %s", strerror(errno));
+	    exit(1);
+	}
+	close(0);
+	close(1);
+	close(2);
+    } else if (ser2net_debug) {
+	openlog("ser2net", LOG_PID | LOG_CONS | LOG_PERROR, LOG_DAEMON);
+    } else {
+	openlog("ser2net", LOG_PID | LOG_CONS, LOG_DAEMON);
+    }
+#endif
 }
 
 static struct gensio_lock *config_lock;
-
 static struct gensio_lock *maint_lock;
 
-#ifdef USE_PTHREADS
 static int in_config_read = 0;
 
-int ser2net_wake_sig = SIGUSR1;
-void (*finish_shutdown)(void);
-
 void
 start_maint_op(void)
 {
-    so->lock(maint_lock);
+    gensio_os_funcs_lock(so, maint_lock);
 }
 
 void
 end_maint_op(void)
 {
-    so->unlock(maint_lock);
-}
-
-static void *
-config_reread_thread(void *dummy)
-{
-    pthread_detach(pthread_self());
-    start_maint_op();
-    reread_config_file("SIGHUP", &syslog_absout);
-    end_maint_op();
-    so->lock(config_lock);
-    in_config_read = 0;
-    so->unlock(config_lock);
-    return NULL;
+    gensio_os_funcs_unlock(so, maint_lock);
 }
 
 static void
-thread_reread_config_file(void)
+cleanup_s2n_threadinfo(struct s2n_threadinfo *thread)
+{
+    if (thread->waiter)
+	gensio_os_funcs_free_waiter(so, thread->waiter);
+    if (thread->runner)
+	gensio_os_funcs_free_runner(so, thread->runner);
+}
+
+static void
+config_join_runner(struct gensio_runner *r, void *data)
+{
+    struct s2n_threadinfo *thread = data;
+
+    gensio_os_wait_thread(thread->gthread);
+    cleanup_s2n_threadinfo(thread);
+    gensio_os_funcs_zfree(so, thread);
+}
+
+static void
+config_reread_thread(void *data)
+{
+    struct s2n_threadinfo *thread = data;
+
+    start_maint_op();
+    reread_config_file("SIGHUP", &seout);
+    end_maint_op();
+    gensio_os_funcs_lock(so, config_lock);
+    in_config_read = 0;
+    gensio_os_funcs_unlock(so, config_lock);
+    gensio_os_funcs_run(so, thread->runner);
+}
+
+static void
+thread_reread_config_file(void *data)
 {
     int rv;
-    pthread_t thread;
+    struct s2n_threadinfo *thread = NULL;
 
-    so->lock(config_lock);
-    if (in_config_read) {
+    gensio_os_funcs_lock(so, config_lock);
+    if (in_config_read || in_shutdown) {
 	so->unlock(config_lock);
 	return;
     }
     in_config_read = 1;
-    so->unlock(config_lock);
+    gensio_os_funcs_unlock(so, config_lock);
 
-    rv = pthread_create(&thread, NULL, config_reread_thread, NULL);
+    thread = gensio_os_funcs_zalloc(so, sizeof(*thread));
+    if (!thread)
+	goto out_nomem;
+    thread->waiter = gensio_os_funcs_alloc_waiter(so);
+    if (!thread->waiter)
+	goto out_nomem;
+    thread->runner = gensio_os_funcs_alloc_runner(so, config_join_runner,
+						  thread);
+    if (!thread->runner)
+	goto out_nomem;
+
+    rv = gensio_os_new_thread(so, config_reread_thread,
+			      thread, &thread->gthread);
     if (rv) {
-	syslog(LOG_ERR,
-	       "Unable to start thread to reread config file: %s",
-	       strerror(rv));
-	so->lock(config_lock);
-	in_config_read = 0;
-	so->unlock(config_lock);
+	seout.out(&seout,
+		  "Unable to start thread to reread config file: %s",
+		  gensio_err_to_str(rv));
+	goto out_unlock;
     }
+    return;
+
+ out_nomem:
+    seout.out(&seout, "Reconfig failed: Out of memory");
+ out_unlock:
+    if (thread)
+	cleanup_s2n_threadinfo(thread);
+    gensio_os_funcs_lock(so, config_lock);
+    in_config_read = 0;
+    gensio_os_funcs_unlock(so, config_lock);
 }
 
 static void
-wake_thread_sighandler(int sig)
+op_loop(void *data)
 {
-    /* Nothing to do, sending the sig just wakes up select(). */
+    struct s2n_threadinfo *thread = data;
+
+    gensio_os_funcs_wait(so, thread->waiter, 1, NULL);
 }
 
-static void
-wake_thread_send_sig(long thread_id, void *cb_data)
-{
-    pthread_t        *id = (void *) thread_id;
-
-    pthread_kill(*id, ser2net_wake_sig);
-}
-
-static void *
-op_loop(void *dummy)
-{
-    pthread_t self = pthread_self();
-
-    while (!in_shutdown)
-	sel_select(ser2net_sel, wake_thread_send_sig, (long) &self, NULL, NULL);
-
-    /* Join the threads only in the first thread.  You cannot join the
-       first thread.  Finish the shutdown in the first thread. */
-    if (self == threads[0].id) {
-	int i;
-
-	for (i = 0; i < num_threads; i++) {
-	    if (threads[i].id == self)
-		continue;
-	    pthread_join(threads[i].id, NULL);
-	}
-	free(threads);
-
-	finish_shutdown();
-    }
-    return NULL;
-}
-
-static void
+static int
 start_threads(void)
 {
     int i, rv;
-    struct sigaction act;
 
-    act.sa_handler = wake_thread_sighandler;
-    sigemptyset(&act.sa_mask);
-    act.sa_flags = 0;
-    rv = sigaction(ser2net_wake_sig, &act, NULL);
-    if (rv) {
-	syslog(LOG_ERR, "Unable to set sigaction: %s", strerror(errno));
-	exit(1);
-    }
-
-    threads = malloc(sizeof(*threads) * num_threads);
+    threads = gensio_os_funcs_zalloc(so, sizeof(*threads) * num_threads);
     if (!threads) {
-	syslog(LOG_ERR, "Unable to allocate thread info");
-	exit(1);
+	seout.out(&seout, "Unable to allocate thread info");
+	return 1;
     }
 
-    threads[0].id = pthread_self();
-
-    for (i = 1; i < num_threads; i++) {
-	rv = pthread_create(&threads[i].id, NULL, op_loop, NULL);
-	if (rv) {
-	    syslog(LOG_ERR, "Unable to start thread: %s", strerror(rv));
-	    exit(1);
-	}
-    }
-}
-
-static void
-stop_threads(void (*finish)(void))
-{
-    int i;
-    pthread_t self = pthread_self();
-
-    in_shutdown = 1;
-    finish_shutdown = finish;
-    start_maint_op();
-    /* Make sure we aren't in a reconfig. */
-    end_maint_op();
-
     for (i = 0; i < num_threads; i++) {
-	if (threads[i].id == self)
-	    continue;
-	pthread_kill(threads[i].id, ser2net_wake_sig);
+	threads[i].waiter = gensio_os_funcs_alloc_waiter(so);
+	if (!threads[i].waiter)
+	    goto out_nomem;
     }
-}
 
-struct sel_lock_s
-{
-    pthread_mutex_t lock;
-};
-
-static sel_lock_t *
-slock_alloc(void *cb_data)
-{
-    sel_lock_t *l;
-
-    l = malloc(sizeof(*l));
-    if (!l)
-	return NULL;
-    pthread_mutex_init(&l->lock, NULL);
-    return l;
-}
-
-static void
-slock_free(sel_lock_t *l)
-{
-    pthread_mutex_destroy(&l->lock);
-    free(l);
-}
-
-static void
-slock_lock(sel_lock_t *l)
-{
-    pthread_mutex_lock(&l->lock);
-}
-
-static void
-slock_unlock(sel_lock_t *l)
-{
-    pthread_mutex_unlock(&l->lock);
-}
-
-#else
-int ser2net_wake_sig = 0;
-void start_maint_op(void) { }
-void end_maint_op(void) { }
-static void start_threads(void) { }
-static void stop_threads(void (*finish)(void)) { finish(); }
-#define slock_alloc NULL
-#define slock_free NULL
-#define slock_lock NULL
-#define slock_unlock NULL
-static void *
-op_loop(void *dummy)
-{
-    sel_select_loop(ser2net_sel, NULL, 0, NULL);
-    return NULL;
-}
-#endif /* USE_PTHREADS */
-
-static void
-finish_shutdown_cleanly(void)
-{
-    struct timeval tv;
-
-    sel_clear_fd_handlers(ser2net_sel, sig_fd_watch);
-    free_rotators();
-    free_controllers();
-    shutdown_ports();
-    do {
-	if (check_ports_shutdown())
-	    break;
-	tv.tv_sec = 1;
-	tv.tv_usec = 0;
-	sel_select(ser2net_sel, NULL, 0, NULL, &tv);
-    } while(1);
-
-    shutdown_dataxfer();
-
-    free_longstrs();
-    free_tracefiles();
-    free_rs485confs();
-
-    if (pid_file)
-	unlink(pid_file);
-
-    if (admin_port)
-	free(admin_port);
-
-    so->free_funcs(so);
-
-    exit(1);
-}
-
-static void
-shutdown_cleanly(void)
-{
-    stop_threads(finish_shutdown_cleanly);
-}
-
-static void
-sig_fd_read_handler(int fd, void *cb_data)
-{
-    char dummy[10];
-
-    dummyrv = read(fd, dummy, sizeof(dummy));
-
-    if (term_prog)
-	shutdown_cleanly();
-
-    if (reread_config && !in_shutdown) {
-#ifdef USE_PTHREADS
-	thread_reread_config_file();
-#else
-	start_maint_op();
-	reread_config_file("SIGHUP", &syslog_absout);
-	end_maint_op();
-#endif
+    for (i = 1; i < num_threads; i++) {
+	rv = gensio_os_new_thread(so, op_loop, &threads[i],
+				  &threads[i].gthread);
+	if (rv) {
+	    seout.out(&seout, "Unable to start thread: %s",
+		      gensio_err_to_str(rv));
+	    goto out;
+	}
     }
-}
-
-static void
-setup_signals(void)
-{
-    struct sigaction act;
-    int              err;
-    int              pipefds[2];
-
-    /* Ignore SIGPIPEs so they don't kill us. */
-    signal(SIGPIPE, SIG_IGN);
-
-    err = pipe(pipefds);
-    if (err)
-	goto out;
-
-    sig_fd_alert = pipefds[1];
-    sig_fd_watch = pipefds[0];
-
-    act.sa_handler = sighup_handler;
-    sigemptyset(&act.sa_mask);
-    act.sa_flags = SA_RESTART;
-    err = sigaction(SIGHUP, &act, NULL);
-    if (err)
-	goto out;
-
-    act.sa_handler = sigint_handler;
-    /* Only handle SIGINT once. */
-    act.sa_flags |= SA_RESETHAND;
-    err = sigaction(SIGINT, &act, NULL);
-    if (!err)
-	err = sigaction(SIGQUIT, &act, NULL);
-    if (!err)
-	err = sigaction(SIGTERM, &act, NULL);
-    if (err)
-	goto out;
-
-    err = sel_set_fd_handlers(ser2net_sel, sig_fd_watch, NULL,
-			      sig_fd_read_handler, NULL, NULL, NULL);
-    if (!err)
-	sel_set_fd_read_handler(ser2net_sel, sig_fd_watch,
-				SEL_FD_HANDLER_ENABLED);
+    return 0;
 
+ out_nomem:
+    seout.out(&seout, "Unable to alloc data for threads");
  out:
-    if (err) {
-	fprintf(stderr, "Error setting up signals: %s\n", strerror(errno));
-	exit(1);
+    for (i = 0; i < num_threads; i++) {
+	if (threads[i].gthread) {
+	    gensio_os_funcs_wake(so, threads[i].waiter);
+	    gensio_os_wait_thread(threads[i].gthread);
+	}
+	cleanup_s2n_threadinfo(&threads[i]);
     }
+    gensio_os_funcs_zfree(so, threads);
+    threads = NULL;
+    return 1;
 }
 
 static void
-ser2net_gensio_logger(struct gensio_os_funcs *o, enum gensio_log_levels level,
-		      const char *log, va_list args)
+stop_threads(void)
 {
-    int priority = LOG_INFO;
+    int i;
 
-    switch (level) {
-    case GENSIO_LOG_FATAL:   priority = LOG_CRIT; break;
-    case GENSIO_LOG_ERR:     priority = LOG_ERR; break;
-    case GENSIO_LOG_WARNING: priority = LOG_WARNING; break;
-    case GENSIO_LOG_INFO:    priority = LOG_INFO; break;
-    case GENSIO_LOG_DEBUG:   priority = LOG_DEBUG; break;
+    for (i = 0; i < num_threads; i++)
+	gensio_os_funcs_wake(so, threads[i].waiter);
+}
+
+static void
+shutdown_cleanly(void *data)
+{
+    struct gensio_time timeout;
+
+    gensio_os_funcs_lock(so, config_lock);
+    in_shutdown = 1;
+    /* Make sure we aren't in a reconfig. */
+    while (in_config_read) {
+	gensio_os_funcs_unlock(so, config_lock);
+	timeout.secs = 1;
+	timeout.nsecs = 0;
+	gensio_os_funcs_service(so, &timeout);
+	gensio_os_funcs_lock(so, config_lock);
     }
+    gensio_os_funcs_unlock(so, config_lock);
 
-    vsyslog(priority, log, args);
+    stop_threads();
 }
 
 int
 main(int argc, char *argv[])
 {
-    int i;
-    int err;
-#ifdef USE_PTHREADS
+    unsigned int i;
+    int err, rv;
     char *end;
-#endif
     int print_when_ready = 0;
-    FILE *instream = NULL;
-    enum { CONFIG_NONE, CONFIG_YAML, CONFIG_OLD } config_type = CONFIG_NONE;
+    ftype *instream = NULL;
     char *filename;
 
     gensio_set_progname("ser2net");
@@ -728,19 +790,15 @@ main(int argc, char *argv[])
     config_lines = malloc(sizeof(*config_lines));
     if (!config_lines) {
 	fprintf(stderr, "Out of memory\n");
-	exit(1);
+	return 1;
     }
     *config_lines = NULL;
 
-    if (led_driver_init() < 0) {
-	fprintf(stderr, "Error while initializing LED drivers\n");
-	exit(1);
-    }
-
     for (i = 1; i < argc; i++) {
 	if ((argv[i][0] != '-') || (strlen(argv[i]) != 2)) {
 	    fprintf(stderr, "Invalid argument: '%s'\n", argv[i]);
 	    arg_error(argv[0]);
+	    return 1;
 	}
 
 	switch (argv[i][1]) {
@@ -766,39 +824,27 @@ main(int argc, char *argv[])
 	case 'b':
 	    break;
 
-	case 'C':
 	case 'Y':
-	    if (argv[i][1] == 'C') {
-		if (config_type == CONFIG_YAML) {
-		mixed_config_lines:
-		    fprintf(stderr,
-			    "You cannot mix yaml and old config lines.\n");
-		    arg_error(argv[0]);
-		}
-		config_type = CONFIG_OLD;
-	    } else {
-		if (config_type == CONFIG_OLD)
-		    goto mixed_config_lines;
-		config_type = CONFIG_YAML;
-	    }
-
 	    /* Get a config line. */
 	    i++;
 	    if (i == argc) {
 		fprintf(stderr, "No config line specified with -%c\n",
 			argv[i][1]);
 		arg_error(argv[0]);
+		return 1;
 	    }
 	    num_config_lines++;
 	    config_lines = realloc(config_lines, sizeof(*config_lines) *
 				   (num_config_lines + 1));
 	    if (!config_lines) {
 		fprintf(stderr, "Out of memory handling config line\n");
-		exit(1);
+		return 1;
 	    }
 	    config_lines[num_config_lines - 1] = argv[i];
-	    config_file = NULL;
-	    config_file_set = true;
+	    if (!config_file_set) {
+		config_file = NULL;
+		config_file_set = true;
+	    }
 	    break;
 
 	case 'c':
@@ -807,22 +853,54 @@ main(int argc, char *argv[])
 	    if (i == argc) {
 		fprintf(stderr, "No config file specified with -c\n");
 		arg_error(argv[0]);
+		return 1;
 	    }
 	    config_file = argv[i];
 	    config_file_set = true;
 	    break;
 
+	case 'C':
+	    i++;
+	    if (i == argc) {
+		fprintf(stderr, "No config dir specified with -c\n");
+		arg_error(argv[0]);
+		return 1;
+	    }
+	    confdir = argv[i];
+	    break;
+
+	case 'a':
+	    i++;
+	    if (i == argc) {
+		fprintf(stderr, "No auth dir specified with -a\n");
+		arg_error(argv[0]);
+		return 1;
+	    }
+	    authdir = argv[i];
+	    break;
+
+	case 'A':
+	    i++;
+	    if (i == argc) {
+		fprintf(stderr, "No admin auth dir specified with -A\n");
+		arg_error(argv[0]);
+		return 1;
+	    }
+	    admin_authdir = argv[i];
+	    break;
+
 	case 'p':
 	    /* Get the control port. */
 	    i++;
 	    if (i == argc) {
 		fprintf(stderr, "No control port specified with -p\n");
 		arg_error(argv[0]);
+		return 1;
 	    }
 	    admin_port = strdup(argv[i]);
 	    if (!admin_port) {
 		fprintf(stderr, "Could not allocate memory for -p\n");
-		exit(1);
+		return 1;
 	    }
 	    admin_port_from_cmdline = true;
 	    break;
@@ -832,6 +910,7 @@ main(int argc, char *argv[])
 	    if (i == argc) {
 		fprintf(stderr, "No pid file specified with -P\n");
 		arg_error(argv[0]);
+		return 1;
 	    }
 	    pid_file = argv[i];
 	    break;
@@ -842,199 +921,182 @@ main(int argc, char *argv[])
 
 	case 'v':
 	    printf("%s version %s\n", argv[0], VERSION);
-	    exit(0);
+	    return 1;
 
 	case 's':
             i++;
             if (i == argc) {
 	        fprintf(stderr, "No signature specified\n");
-		exit(1);
+		return 1;
             }
             rfc2217_signature = argv[i];
             break;
 
-#ifdef USE_PTHREADS
 	case 't':
             i++;
             if (i == argc) {
 	        fprintf(stderr, "No thread count specified\n");
-		exit(1);
+		return 1;
             }
 	    num_threads = strtoul(argv[i], &end, 10);
 	    if (end == argv[i] || *end != '\0') {
 	        fprintf(stderr, "Invalid thread count specified: %s\n",
 			argv[i]);
-		exit(1);
+		return 1;
 	    }
+	    if (num_threads == 0)
+		num_threads = 1;
             break;
-#endif
 
 	default:
 	    fprintf(stderr, "Invalid option: '%s'\n", argv[i]);
 	    arg_error(argv[0]);
+	    return 1;
 	}
     }
 
-#ifdef USE_PTHREADS
-    if (num_threads > 1)
-	err = sel_alloc_selector_thread(&ser2net_sel, ser2net_wake_sig,
-					slock_alloc, slock_free,
-					slock_lock, slock_unlock, NULL);
-    else
-#endif
-	err = sel_alloc_selector_nothread(&ser2net_sel);
+    if (setup_paths())
+	return 1;
 
+    err = gensio_default_os_hnd(SIGUSR1, &so);
     if (err) {
-	fprintf(stderr,
-		"Could not initialize ser2net selector: '%s'\n",
-		strerror(err));
-	exit(1);
-    }
-
-    so = gensio_selector_alloc(ser2net_sel, ser2net_wake_sig);
-    if (!so) {
 	fprintf(stderr, "Could not alloc ser2net gensio selector\n");
-	exit(1);
+	return 1;
     }
     so->vlog = ser2net_gensio_logger;
 
-    config_lock = so->alloc_lock(so);
+    err = gensio_os_proc_setup(so, &procdata);
+    if (err) {
+	seout.out(&seout, "Unable to setup proc: %s", gensio_err_to_str(err));
+	return 1;
+    }
+
+    err = gensio_os_proc_register_reload_handler(procdata,
+						 thread_reread_config_file,
+						 NULL);
+    if (err && err != GE_NOTSUP) {
+	seout.out(&seout, "Unable to setup reconfig handler: %s",
+		  gensio_err_to_str(err));
+	return 1;
+    }
+
+    err = gensio_os_proc_register_term_handler(procdata, shutdown_cleanly,
+					       NULL);
+    if (err) {
+	seout.out(&seout, "Unable to setup termination handler: %s",
+		  gensio_err_to_str(err));
+	return 1;
+    }
+
+    if (led_driver_init() < 0) {
+	fprintf(stderr, "Error while initializing LED drivers\n");
+	return 1;
+    }
+
+    config_lock = gensio_os_funcs_alloc_lock(so);
     if (!config_lock) {
 	fprintf(stderr, "Could not alloc ser2net config lock\n");
-	exit(1);
+	return 1;
     }
 
-    maint_lock = so->alloc_lock(so);
+    maint_lock = gensio_os_funcs_alloc_lock(so);
     if (!maint_lock) {
 	fprintf(stderr, "Could not alloc ser2net maint lock\n");
-	exit(1);
+	return 1;
     }
 
-    setup_signals();
+    init_mdns();
 
     err = init_dataxfer();
     if (err) {
 	fprintf(stderr,
 		"Could not initialize dataxfer: '%s'\n",
 		strerror(err));
-	exit(1);
+	return 1;
     }
 
-    if (ser2net_debug && !detach)
-	openlog("ser2net", LOG_PID | LOG_CONS | LOG_PERROR, LOG_DAEMON);
-
     err = readconfig_init();
     if (err) {
 	fprintf(stderr,
 		"Could not initialize defaults: '%s'\n",
 		gensio_err_to_str(err));
-	exit(1);
+	return 1;
     }
 
     if (admin_port)
-	controller_init(admin_port, NULL, NULL);
+	controller_init(admin_port, NULL, NULL, &stderr_absout);
 
-    if (config_type == CONFIG_OLD) {
-	for (i = 0; i < num_config_lines; i++)
-	    handle_config_line(config_lines[i], strlen(config_lines[i]));
-    }
     if (config_file) {
-	bool is_yaml;
-
 	if (strcmp(config_file, "-") == 0) {
-	    instream = stdin;
-	    is_yaml = true;
+	    err = f_stdio_open(stdin, DO_READ, 0, &instream);
+	    if (err) {
+		fprintf(stderr, "Unable to create stdin file: %s\n",
+			gensio_err_to_str(err));
+		return 1;
+	    }
 	    filename = "<stdin>";
 	} else {
-	    instream = fopen_config_file(&is_yaml, &filename);
+	    instream = fopen_config_file(&filename, &stderr_absout);
 	    if (!instream)
-		exit(1);
-	}
-	if (is_yaml) {
-	    if (config_type == CONFIG_OLD) {
-		fprintf(stderr, "You cannot mix old style config lines "
-			"and a yaml config file.\n");
-		exit(1);
-	    }
-	    config_type = CONFIG_YAML;
-	} else {
-	    if (config_type == CONFIG_YAML) {
-		fprintf(stderr, "You cannot mix yaml config lines "
-			"and an old style config file.\n");
-		exit(1);
-	    }
-	    config_type = CONFIG_OLD;
+		return 1;
 	}
     } else {
 	filename = "<cmdline>";
     }
 
-    if (config_type == CONFIG_YAML || instream)	{
-	int rv;
+    rv = yaml_readconfig(instream, filename,
+			 config_lines, num_config_lines,
+			 &stderr_absout);
+    if (rv)
+	return 1;
+    if (instream)
+	f_close(instream);
 
-	if (config_type == CONFIG_YAML)
-	    rv = yaml_readconfig(instream, filename,
-				 config_lines, num_config_lines,
-				 &stderr_absout);
-	else
-	    rv = readconfig(instream);
-	if (rv == -1)
-	    exit(1);
-	if (instream && instream != stdin)
-	    fclose(instream);
-    }
-    apply_new_ports(&syslog_absout);
+    apply_new_ports(&seout);
 
-    if (detach) {
-	int pid;
-
-	/* Detach from the calling terminal. */
-	openlog("ser2net", LOG_PID | LOG_CONS, LOG_DAEMON);
-	syslog(LOG_NOTICE, "ser2net startup");
-	if ((pid = fork()) > 0) {
-	    exit(0);
-	} else if (pid < 0) {
-	    syslog(LOG_ERR, "Error forking first fork: %s", strerror(errno));
-	    exit(1);
-	} else {
-	    /* setsid() is necessary if we really want to demonize */
-	    setsid();
-	    /* Second fork to really deamonize me. */
-	    if ((pid = fork()) > 0) {
-		exit(0);
-	    } else if (pid < 0) {
-		syslog(LOG_ERR, "Error forking second fork: %s",
-		       strerror(errno));
-		exit(1);
-	    }
-	}
-
-	/* Close all my standard I/O. */
-	if (chdir("/") < 0) {
-	    syslog(LOG_ERR, "unable to chdir to '/': %s", strerror(errno));
-	    exit(1);
-	}
-	close(0);
-	close(1);
-	close(2);
-    }
+    do_detach();
 
     /* write pid file */
-    make_pidfile();
+    if (make_pidfile())
+	return 1;
 
-    start_threads();
+    if (start_threads())
+	return 1;
 
     if (print_when_ready) {
 	printf("Ready\n");
 	fflush(stdout);
     }
 
-    op_loop(NULL);
+    op_loop(&threads[0]);
 
-    so->free_lock(maint_lock);
-    so->free_lock(config_lock);
-    so->free_funcs(so);
-    sel_free_selector(ser2net_sel);
+    for (i = 1; i < num_threads; i++)
+	gensio_os_wait_thread(threads[i].gthread);
+
+    free_rotators();
+    free_controllers();
+    shutdown_ports();
+    while (!check_ports_shutdown()) {
+	struct gensio_time timeout;
+
+	timeout.secs = 1;
+	timeout.nsecs = 0;
+	gensio_os_funcs_service(so, &timeout);
+    }
+
+    shutdown_dataxfer();
+
+    cleanup_pidfile();
+
+    if (admin_port)
+	free(admin_port);
+
+    if (config_lines)
+	free(config_lines);
+
+    gensio_os_funcs_free_lock(so, maint_lock);
+    gensio_os_funcs_free_lock(so, config_lock);
+    gensio_os_funcs_free(so);
 
     return 0;
 }
