@@ -12,9 +12,9 @@ test_transfer("basic tcp", "This is a test!",
 test_write_drain("basic tcp", "This is a write drain test!",
                  ("connection: &con",
                   "  accepter: tcp,3023",
-                  "  connector: serialdev,/dev/ttyPipeA0,9600N81"),
+                  "  connector: serialdev,/dev/ttyPipeA0,9600N81,local"),
                  "tcp,localhost,3023",
-                 "serialdev,/dev/ttyPipeB0,9600N81")
+                 "serialdev,/dev/ttyPipeB0,9600N81,local")
 
 test_connect_back("basic tcp", "TCP Connect back test!",
                  ("connection: &con",
