@@ -100,6 +100,7 @@ class HandleData:
         self.to_compare = None
         self.to_waitfor = None
         self.expecting_modemstate = False
+        self.expecting_modemstate_mask = False
         self.expecting_linestate = False
         self.expected_server_cb = None
         self.expected_server_value = 0
@@ -216,7 +217,10 @@ class HandleData:
             return 0
 
         if (err):
-            raise HandlerException(self.name + ": read: " + err)
+            # A remote close is fairly normal, don't error on it.
+            if err != "Remote end closed connection":
+                raise HandlerException(self.name + ": read: " + err)
+            return 0
         if (self.ignore_input):
             return len(buf)
         if (self.to_waitfor):
@@ -310,6 +314,25 @@ class HandleData:
         self.expected_modemstate = modemstate
         return
 
+    def modemstate_mask(self, io, modemstate):
+        if (not self.expecting_modemstate_mask):
+            if (debug or self.debug):
+                print("Got unexpected modemstate_mask for %s: %x" %
+                      (self.name, modemstate))
+            return
+        if (modemstate != self.expected_modemstate_mask):
+            raise HandlerException("%s: Expecting modemstate_mask 0x%x, got 0x%x" %
+                                   (self.name, self.expected_modemstate_mask,
+                                    modemstate))
+        self.expecting_modemstate_mask = False
+        self.waiter.wake()
+        return
+
+    def set_expected_modemstate_mask(self, modemstate):
+        self.expecting_modemstate_mask = True
+        self.expected_modemstate_mask = modemstate
+        return
+
     def linestate(self, io, linestate):
         if (not self.expecting_linestate):
             if (debug or self.debug):
@@ -602,7 +625,7 @@ class Ser2netDaemon:
             else:
                 self.signal(signal.SIGKILL)
             # It would be really nice if waitpid had a timeout options,
-            # in absense of that simulate it, sort of.
+            # in absence of that simulate it, sort of.
             subcount = 500
             while (subcount > 0):
                 time.sleep(.01)
